/*
   htc.c - Harvard to Cat: Test version for both DEK and NDK formats 
   v 1.02b
   Harvard cmt catalogue transformer
   (c) Vladimir Lutikov, 2005-2006
   mailto:darkowl@inbox.ru
   Using some algorithm of Hrvd2cat.c, (c) 1998 by Tikhonov Anthone
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>
#include <string.h>

#define LINESZ 256
#define FALSE 0
#define TRUE  1

typedef struct
{
  short yr, mt, dt, hr, mn;
  float sc;
  float lat, lon, dep;
  float ms, mb;
  short ex;
  float cmtv[6]; /* cmt values [Mrr, Mss, Mee, Mrs, Mre, Mse] */
  float cmtr[6]; /* cmt errors */
  float mo;      /* scalar moment */
} EVENT;

EVENT ev;
char sLine[LINESZ];
long nLine = 0;
int bNewFormat = FALSE; 

int die(void)
{
  sLine[50] = 0;
  printf("Error reading line %ld: %s\n", nLine, sLine);
  getch();
  exit(1);
  return 0;
}

int readline(FILE *f, char *s)
{
  nLine++;
  return fgets(s, LINESZ, f) != NULL;
}

/* convert n chars from s to int */
int sgeti(const char *s, int n)
{
  char buf[32];
  int res;
  strncpy(buf, s, n);
  buf[n] = 0;
  sscanf(buf, "%d", &res) == 1 || die();
  return res;
}

float sgetf(const char *s, int n)
{
  char buf[32];
  float res;
  strncpy(buf, s, n);
  buf[n] = 0;
  sscanf(buf, "%f", &res) == 1 || die();
  return res;
}

void sreadcmt(const char *s, float *pCmtv, float *pCmtr)
{
  int i;
  for(i=0; i<6; i++)
  {
    sscanf(s, "%f %f", &pCmtv[i], &pCmtr[i]) == 2 || die();
    s += bNewFormat ? 13: 11; // +=11 for older (*.dek) version
  }
}

void writecmt(FILE *f, float *pCmt)
{
  int i;
  for(i=0; i<6; i++)
  {
    if ( bNewFormat )
    {
      fprintf(f, "%6.3f ", pCmt[i]);
    } else {
      fprintf(f, "%6.2f ",pCmt[i]);
    }    
  }
}

/* return non zero on success */
int readev(FILE *f, EVENT *e)
{
  if(! readline(f, sLine) ) return 0;
  memset(e, 0, sizeof(*e));
  bNewFormat = (sLine[4] == ' ');
  if (bNewFormat)
  {
	  sscanf(sLine, "%*s %hd/%hd/%hd", &e->yr, &e->mt, &e->dt) == 3 || die();
	   
	  e->hr = sgeti(sLine+16, 2);
	  e->mn = sgeti(sLine+19, 2);
	  e->sc = sgetf(sLine+22, 4);
	
	  e->lat= sgetf(sLine+27, 7);
	  e->lon= sgetf(sLine+34, 8);
	  e->dep= sgetf(sLine+42, 5);
	  e->mb = sgetf(sLine+47, 4);
	  e->ms = sgetf(sLine+51, 4);
	
	  if(! readline(f, sLine) ) return 0;
	  if(! readline(f, sLine) ) return 0; // Skip 2nd and 3rd lines
	  if(! readline(f, sLine) ) return 0;
	  
	  e->ex = sgeti(sLine, 2)-7; /* -7 to convert dyne/sm to N/m */
	  sreadcmt(sLine+2, e->cmtv, e->cmtr);
	  if(! readline(f, sLine) ) return 0;
	  e->mo = sgetf(sLine+49, 7);
  } else {
	  sscanf(sLine, "%*s %hd/%hd/%hd", &e->mt, &e->dt, &e->yr) == 3 || die();
	  e->yr += (e->yr < 30) ? 2000: 1900;
	  	  
	  e->hr = sgeti(sLine+18, 2);
	  e->mn = sgeti(sLine+21, 2);
	  e->sc = sgetf(sLine+24, 4);
	
	  e->lat= sgetf(sLine+29, 7);
	  e->lon= sgetf(sLine+36, 8);
	  e->dep= sgetf(sLine+44, 5);
	  e->mb = sgetf(sLine+49, 3);
	  e->ms = sgetf(sLine+52, 3);
	
	  if(! readline(f, sLine) ) return 0;
	  if(! readline(f, sLine) ) return 0;
	  e->ex = sgeti(sLine+11, 3) - 7;
	  sreadcmt(sLine+14, e->cmtv, e->cmtr);
	  if(! readline(f, sLine) ) return 0;
	  e->mo = sgetf(sLine+42, 7);
  }
  return 1;
}

void writeev(FILE *f, EVENT *e)
{
  fprintf(f, "%4d %2d %2d %2d %2d %4.1f ",
    e->yr, e->mt, e->dt, e->hr, e->mn, e->sc);
  fprintf(f, "%6.2f %7.2f %4.0f %4.1f %4.1f %2d ",
    e->lat, e->lon, e->dep, e->ms, e->mb, e->ex);
  writecmt(f, e->cmtv);
  writecmt(f, e->cmtr);
  if (bNewFormat)
  {
    fprintf(f, "%5.3f\n", e->mo);
  } else {
    fprintf(f, "%5.2f\n", e->mo);
  }    
}

void writehead(FILE *f)
{
  fprintf(f, "Year mt dt hr mn sec  lat    lon     dep  ms   mb   ex ");
  fprintf(f, "Mrr    Mss    Mee    Mrs    Mre    Mse    ");
  fprintf(f, "EMrr   EMss   EMee   EMrs   EMre   EMse   Mo\n");
}

int main (int argc, char * argv[])
{
  FILE *fi, *fo;
  long i = 0;

  if (argc < 3)
  {
    printf("Harvard CMT catalogue (*.dek, *.ndk) transformer\nv 1.02b\n");
    printf("Convert DEK and NDK files to Excel-readable table .)\n\n");
    
    printf("Usage: HTC infile outfile\n");
    getch();
    return 1;
  }

  if( (fi=fopen(argv[1],"r")) == NULL )
  {
    printf("Unable to open file '%s'.\n", argv[1]);
    getch();
    return 1;
  }
  if( (fo = fopen(argv[2],"w")) == NULL )
  {
    printf("Unable to write file '%s'.\n", argv[2]);
    getch();
    return 2;
  }
  printf("Run...\n");
  writehead(fo);
  while( readev(fi, &ev) )
  {
    i++;
    writeev(fo, &ev);
  }
  fclose(fi);
  fclose(fo);
  printf("Done.\n%ld events print\n", i);
  getch();
  printf("Copyright (c) 2005-2006, by Vladimir Lutikov\n");
  return 0;
}
